#include <built_in.h>

// MP3 connections
sbit Mmc_Chip_Select           at LATG9_bit;
sbit MP3_CS                    at RG12_bit;
sbit MP3_RST                   at RG13_bit;
sbit DREQ                      at RG1_bit;
sbit BSYNC                     at RG14_bit;
sbit DCLK                      at LATG0_bit;
sbit SDATA                     at RA6_bit;

sbit Mmc_Chip_Select_Direction at TRISG9_bit;
sbit MP3_CS_Direction          at TRISG12_bit;
sbit MP3_RST_Direction         at TRISG13_bit;
sbit DREQ_Direction            at TRISG1_bit;
sbit BSYNC_Direction           at TRISG14_bit;
sbit DCLK_Direction            at TRISG0_bit;
sbit SDATA_Direction           at TRISA6_bit;

// VS1011E constants
const char WRITE_CODE  = 0x02;
const char READ_CODE   = 0x03;
const char MODE_ADDR   = 0x00;
const char CLOCKF_ADDR = 0x03;
const char VOL_ADDR    = 0x0B;

char  data_buffer_32[32];

// Writes one byte to MP3 SDI
static void SW_SPI_Write(char data_) {
  BSYNC = 1;                              // Set BSYNC before sending the first bit

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;                            // bitorder is LSB first

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  BSYNC = 0;                              // Clear BSYNC after sending the second bit

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
}

// Writes one word to MP3 SCI
static void MP3_SCI_Write(char address, unsigned int data_in) {
  MP3_CS = 0;                             // select MP3 SCI
  SPI2_Write(WRITE_CODE);
  SPI2_Write(address);
  SPI2_Write(Hi(data_in));                // high byte
  SPI2_Write(Lo(data_in));                // low byte
  MP3_CS = 1;                             // deselect MP3 SCI
  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

// Reads words_count words from MP3 SCI
static void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer) {
  unsigned int temp;

  MP3_CS = 0;                             // select MP3 SCI
  SPI2_Write(READ_CODE);
  SPI2_Write(start_address);

  while (words_count--) {                 // read words_count words byte per byte
    temp = SPI2_Read(0);
    temp <<= 8;
    temp += SPI2_Read(0);
    *(data_buffer++) = temp;
  }
  MP3_CS = 1;                             // deselect MP3 SCI
  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

// Write one byte to MP3 SDI
void MP3_SDI_Write(char data_) {
  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  SW_SPI_Write(data_);
}

// Write 32 bytes to MP3 SDI
void MP3_SDI_Write_32(char *data_) {
  char i;

  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  for (i=0; i<32; i++)
    SW_SPI_Write(data_[i]);
}

// Set clock
static void Set_Clock(unsigned int clock_khz, char doubler) {
  clock_khz /= 2;                         // Calculate value
  if (doubler) {
    clock_khz |= 0x8000;
  }
  MP3_SCI_Write(CLOCKF_ADDR, clock_khz);  // Write value to CLOCKF register
}

// Set volume
void Set_Volume(char left,char right) {
  unsigned int volume;

  volume = ((255-left) << 8) + (255 - right);             // Calculate value
  MP3_SCI_Write(VOL_ADDR, volume);        // Write value to VOL register
}
// If volume Buttons are pressed update volume


// Software Reset
static void Soft_Reset(char fill_zeros) {
  char i;

  MP3_SCI_Write(MODE_ADDR,0x0204);        // Write to MODE register
                                          // Set SM_RESET bit and SM_BITORD bit(bitorder is LSB first)
  Delay_us(2);                            // Required, see MP3 codec datasheet -> Software Reset
  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  if (fill_zeros) {                       // feed 2048 zeros to the MP3 SDI bus:
    for (i=0; i<32; i++) {                // clear 32byte buffer
      data_buffer_32[i] = 0;
    }
    for (i=0; i < 2048/32; i++) {         // use 32byte buffer to clear MP3 buffer
      MP3_SDI_Write_32(data_buffer_32);
    }
  }
  else
    MP3_SDI_Write(0);                     // write zero once
}

// Initialize VS1011E
void MP3_reset(char volume_left, char volume_right) {
  Soft_Reset(0);                          // SW Reset, do not feed zeros
  Set_Clock(25000,0);                     // Set clock to 25MHz, do not use clock doubler
  Set_Volume(volume_left, volume_right);
}

void MP3_init() {
  ADPCFG = 0xFFFF;        // set all AN pins to digital

  DCLK_Direction  = 0;                    // Set SW SPI pin directions to output
  SDATA_Direction = 0;
  DCLK  = 0;                              // Clear SW SPI SCK
  SDATA = 0;                              // Clear SW SPI SDO

  MP3_CS_Direction  = 0;                  // Configure MP3_CS as output
  MP3_CS            = 1;                  // Deselect MP3_CS
  MP3_RST_Direction = 0;                  // Configure MP3_RST as output
  MP3_RST           = 1;                  // Set MP3_RST pin

  DREQ_Direction  = 1;                    // Configure DREQ as input
  BSYNC_Direction = 0;                    // Configure BSYNC as output
  BSYNC           = 0;                    // Clear BSYNC
  MP3_reset(50, 50);
}